/*
  ==============================================================================

    This file was auto-generated!

    It contains the basic framework code for a JUCE plugin processor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
TestProjectAudioProcessor::TestProjectAudioProcessor()
#ifndef JucePlugin_PreferredChannelConfigurations
     : AudioProcessor (BusesProperties()
                     #if ! JucePlugin_IsMidiEffect
                      #if ! JucePlugin_IsSynth
                       .withInput  ("Input",  AudioChannelSet::stereo(), true)
                      #endif
                       .withOutput ("Output", AudioChannelSet::stereo(), true)
                     #endif
                       ),
    state(*this, nullptr, "state", createParameterLayout())
#endif
{
    state.addParameterListener("LP", this);
    state.addParameterListener("HP", this);

    mHighpassFilterL.reset(new juce::dsp::StateVariableFilter::Filter<float>());
    mHighpassFilterR.reset(new juce::dsp::StateVariableFilter::Filter<float>());

    mLowpassFilterL.reset(new juce::dsp::StateVariableFilter::Filter<float>());
    mLowpassFilterR.reset(new juce::dsp::StateVariableFilter::Filter<float>());

    isActive.store(false);
    mustUpdateProcessing.store(false);
}

TestProjectAudioProcessor::~TestProjectAudioProcessor()
{
}

AudioProcessorValueTreeState::ParameterLayout TestProjectAudioProcessor::createParameterLayout()
{
    std::vector<std::unique_ptr<RangedAudioParameter>> params;

    params.push_back(std::make_unique<AudioParameterFloat>("LP", "Low-pass cutoff", NormalisableRange<float>(20.f, 20000.f, 0.f, 0.5f), 20000.f, "Hz"));
    params.push_back(std::make_unique<AudioParameterFloat>("HP", "High-pass cutoff", NormalisableRange<float>(20.f, 20000.f, 0.f, 0.5f), 20.f, "Hz"));

    return { params.begin(), params.end() };
}

//==============================================================================
void TestProjectAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBlock)
{
    dsp::ProcessSpec spec{ sampleRate, (uint32)samplesPerBlock, 1 };
    
    mHighpassFilterL->prepare(spec);
    mHighpassFilterL->parameters->type = juce::dsp::StateVariableFilter::Parameters<float>::Type::highPass;

    mHighpassFilterR->prepare(spec);
    mHighpassFilterR->parameters->type = juce::dsp::StateVariableFilter::Parameters<float>::Type::highPass;
    
    mLowpassFilterL->prepare(spec);
    mLowpassFilterL->parameters->type = juce::dsp::StateVariableFilter::Parameters<float>::Type::lowPass;
    
    mLowpassFilterR->prepare(spec);
    mLowpassFilterR->parameters->type = juce::dsp::StateVariableFilter::Parameters<float>::Type::lowPass;
    
    updateProcessing();
    reset();

    isActive.store(true);
}

void TestProjectAudioProcessor::releaseResources()
{
    
}

void TestProjectAudioProcessor::reset()
{
    mLowpassFilterL->reset();
    mLowpassFilterR->reset();

    mHighpassFilterL->reset();
    mHighpassFilterR->reset();
}

#ifndef JucePlugin_PreferredChannelConfigurations
bool TestProjectAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
{
  #if JucePlugin_IsMidiEffect
    ignoreUnused (layouts);
    return true;
  #else
    // This is the place where you check if the layout is supported.
    // In this template code we only support mono or stereo.
    if (layouts.getMainOutputChannelSet() != AudioChannelSet::mono()
     && layouts.getMainOutputChannelSet() != AudioChannelSet::stereo())
        return false;

    // This checks if the input layout matches the output layout
   #if ! JucePlugin_IsSynth
    if (layouts.getMainOutputChannelSet() != layouts.getMainInputChannelSet())
        return false;
   #endif

    return true;
  #endif
}
#endif

void TestProjectAudioProcessor::processBlock (AudioBuffer<float>& buffer, MidiBuffer& midiMessages)
{
    ignoreUnused(midiMessages);
    
    if (isActive.load() == false)
        return;
    
    if (mustUpdateProcessing.exchange(false))
        updateProcessing();
    
    ScopedNoDenormals noDenormals;
    auto totalNumInputChannels  = getTotalNumInputChannels();
    auto totalNumOutputChannels = getTotalNumOutputChannels();
    auto numChannels = jmin(totalNumInputChannels, totalNumOutputChannels);

    dsp::AudioBlock<float> leftBlock(buffer);
    leftBlock = leftBlock.getSingleChannelBlock(0);

    mHighpassFilterL->process(dsp::ProcessContextReplacing<float>(leftBlock));
    mLowpassFilterL->process(dsp::ProcessContextReplacing<float>(leftBlock));

    if (numChannels >= 2)
    {
        dsp::AudioBlock<float> rightBlock(buffer);
        rightBlock = rightBlock.getSingleChannelBlock(1);

        mHighpassFilterR->process(dsp::ProcessContextReplacing<float>(rightBlock));
        mLowpassFilterR->process(dsp::ProcessContextReplacing<float>(rightBlock));
    }
}

void TestProjectAudioProcessor::updateProcessing()
{    
    auto mapped_LowpassCutoff = *(state.getRawParameterValue("LP"));
    auto mapped_HighpassCutoff = *(state.getRawParameterValue("HP"));
    
    mLowpassFilterL->parameters->setCutOffFrequency(getSampleRate(), mapped_LowpassCutoff);
    mLowpassFilterR->parameters->setCutOffFrequency(getSampleRate(), mapped_LowpassCutoff);
    
    mHighpassFilterL->parameters->setCutOffFrequency(getSampleRate(), mapped_HighpassCutoff);
    mHighpassFilterR->parameters->setCutOffFrequency(getSampleRate(), mapped_HighpassCutoff);
}

//==============================================================================
bool TestProjectAudioProcessor::hasEditor() const
{
    return true; // (change this to false if you choose to not supply an editor)
}

AudioProcessorEditor* TestProjectAudioProcessor::createEditor()
{
    return new TestProjectAudioProcessorEditor (*this);
}

//==============================================================================
void TestProjectAudioProcessor::getStateInformation (MemoryBlock& destData)
{
    ValueTree copyState = state.copyState();
    ScopedPointer<XmlElement> xml = copyState.createXml();
    copyXmlToBinary(*xml, destData);
}

void TestProjectAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
{
    ScopedPointer<XmlElement> xml = getXmlFromBinary(data, sizeInBytes);
    ValueTree copyState = ValueTree::fromXml(*xml);
    state.replaceState(copyState);
}

//==============================================================================
// This creates new instances of the plugin..
AudioProcessor* JUCE_CALLTYPE createPluginFilter()
{
    return new TestProjectAudioProcessor();
}
